@@ -24,11 +24,11 @@ from .decorators import pass_context, pass_obj, make_pass_decorator, \
      password_option, version_option, help_option
 
 # Types
-from .types import ParamType, File, Choice, IntRange, STRING, INT, FLOAT, \
-     BOOL, UUID
+from .types import ParamType, File, Path, Choice, IntRange, STRING, INT, \
+     FLOAT, BOOL, UUID
 
 # Utilities
-from .utils import echo
+from .utils import echo, get_binary_stream, get_text_stream
 
 # Terminal functions
 from .termui import prompt, confirm, get_terminal_size, echo_via_pager
@@ -54,11 +54,11 @@ __all__ = [
     'version_option', 'help_option',
 
     # Types
-    'ParamType', 'File', 'Choice', 'IntRange', 'STRING', 'INT', 'FLOAT',
-    'BOOL', 'UUID',
+    'ParamType', 'File', 'Path', 'Choice', 'IntRange', 'STRING', 'INT',
+    'FLOAT', 'BOOL', 'UUID',
 
     # Utilities
-    'echo',
+    'echo', 'get_binary_stream', 'get_text_stream',
 
     # Terminal functions
     'prompt', 'confirm', 'get_terminal_size', 'echo_via_pager',
